// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: API Complete
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import java.awt.Dimension;
import java.awt.event.*;
import javax.microedition.midlet.*;
import org.me4se.impl.lcdui.*;
import org.kobjects.scm.*;

public abstract class Canvas extends Displayable {

    public static final int DOWN = 6;
    public static final int LEFT = 2;
    public static final int RIGHT = 5;
    public static final int UP = 1;
    public static final int FIRE = 8;
    public static final int GAME_A = 9;
    public static final int GAME_B = 10;
    public static final int GAME_C = 11;
    public static final int GAME_D = 12;

    public static final int KEY_NUM0 = 48;
    public static final int KEY_NUM1 = 49;
    public static final int KEY_NUM2 = 50;
    public static final int KEY_NUM3 = 51;
    public static final int KEY_NUM4 = 52;
    public static final int KEY_NUM5 = 53;
    public static final int KEY_NUM6 = 54;
    public static final int KEY_NUM7 = 55;
    public static final int KEY_NUM8 = 56;
    public static final int KEY_NUM9 = 57;

    public static final int KEY_POUND = '#';
    public static final int KEY_STAR = '*';


    CanvasComponent component = new CanvasComponent(this);

    public Canvas() {
        Display.check();

        container =
            new DisplayableContainer(
                this,
                null,
                !"true".equals(
                    ApplicationManager.manager.properties.get(
                        "canvas.hideSoftButtons")));
        container.setMain(component);
    }

    void _showNotify() {
        component.requestFocus();
        showNotify();
        repaint(); // sets repaintPending to true
    }

    protected abstract void paint(Graphics g);

    public int getGameAction(int keyCode) {
        //if ((keyCode & 0x08000) != 0)
        // keyCode = keyCode | 0x0ffff0000;
        //System.out.println ("decode: "+keyCode);
        switch (keyCode) {

            case -59 :
            case -KeyEvent.VK_UP - 100 :
                return UP;

            case -60 :
            case -KeyEvent.VK_DOWN - 100 :
                return DOWN;

            case -61 :
            case -KeyEvent.VK_LEFT - 100 :
                return LEFT;

            case -62 :
            case -KeyEvent.VK_RIGHT - 100 :
                return RIGHT;

            case -1 :
            case -4 :
            case -11 :
            case 32 :
                return FIRE;

            case 'a' :
            case 'A' :
            case -KeyEvent.VK_F1 - 100 :
                return GAME_A;

            case 's' :
            case 'S' :
            case -KeyEvent.VK_F2 - 100 :
                return GAME_B;

            case 'd' :
            case 'D' :
                return GAME_C;

            case 'f' :
            case 'F' :
                return GAME_D;

            default :
                return 0;
        }
    }

    public int getKeyCode(int game) {
        switch (game) {
            case UP :
                return -KeyEvent.VK_UP;
            case DOWN :
                return -KeyEvent.VK_DOWN;
            case LEFT :
                return -KeyEvent.VK_LEFT;
            case RIGHT :
                return -KeyEvent.VK_RIGHT;
            case FIRE :
                return 32;
            case GAME_A :
                return 'a';
            case GAME_B :
                return 's';
            case GAME_C :
                return 'd';
            case GAME_D :
                return 'f';
        }
        throw new IllegalArgumentException();
    }

    public String getKeyName(int keyCode) {
        return (keyCode > 0)
            ? ("" + (char) keyCode)
            : KeyEvent.getKeyText(-keyCode);
    }

    public int getHeight() {
        return component.getHeight();
    }

    public int getWidth() {
        return component.getWidth();
    }

    public boolean hasPointerEvents() {
        return !"false".equalsIgnoreCase(
            (String) ApplicationManager.manager.properties.get("touch_screen"));
    }

    public boolean hasPointerMotionEvents() {
        return hasPointerEvents();
    }

    public boolean hasRepeatEvents() {
        return true;
    }

    public boolean isDoubleBuffered() {
        return true;
        /*!"false".equalsIgnoreCase 
        	    ((String) ApplicationManager.manager.properties.get 
        	    ("screenDoubleBuffer"));*/
    }

    protected void hideNotify() {
    }

    void handleCommand (Command cmd) {
        if (cmd == container.GAME_COMMAND1) 
            keyPressed(-1);
        else if (cmd == container.GAME_COMMAND2) {
            keyPressed(-4);
        }
        else super.handleCommand (cmd);
    }


    protected void keyPressed(int code) {
    }

    protected void keyReleased(int code) {
    }

    protected void keyRepeated(int code) {
    }

    protected void pointerDragged(int x, int y) {
    }

    protected void pointerPressed(int x, int y) {
    }

    protected void pointerReleased(int x, int y) {
    }

    public void repaint() {
//        	System.out.println ("repaint called");

        component.repaintPending = true;
        component.repaint();
    }
    /*
    	void sendCommand(Command cmd) {
    		request = true;
    		super.sendCommand(cmd);
    	}
    */
    public void serviceRepaints() {
        if (component.repaintPending)
            synchronized (component.repaintLock) {
                java.awt.Graphics g = component.getGraphics();
                if (g != null)
                    component.paint(g);
            }
    }

    public void repaint(int x, int y, int w, int h) {
        component.repaintPending = true;
        component.repaint(x, y, w, h);
    }

    /*
    	public void setFocus() {
    		request = true;
    		wrapper.requestFocus();
    		ApplicationManager.manager.canvas = wrapper;
    
    		repaintPending = true;
    		wrapper.paint(wrapper.getGraphics());
    	}
    */

    protected void showNotify() {
    }
}