package javax.microedition.lcdui;

/**
 * @author Stefan Haustein
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 */

import org.kobjects.scm.*;
import org.me4se.impl.lcdui.*;
import javax.microedition.midlet.*;
import java.awt.event.*;
import java.util.Vector;

class DisplayableContainer extends ScmContainer {

    static String[] TYPES =
        {
            "null",
            "SCREEN",
            "BACK",
            "CANCEL",
            "OK",
            "HELP",
            "STOP",
            "EXIT",
            "ITEM" };
    static final Command MENU_COMMAND = new Command("Options", 0, 0);
    static final Command SELECT_COMMAND = new Command("Select", 0, 0);
    static final Command GAME_COMMAND1 = new Command("Action1", 0, 0);
    static final Command GAME_COMMAND2 = new Command("Action2", 0, 0);

    Displayable displayable;
    SoftButton[] buttons;
    DeviceLabel title;
    ScmComponent main;
    boolean showButtons;

    /** 
     * This component makes sure that the size of main
     * is allways correct, regardles of the "displayed"
     * status.
     */

    DisplayableContainer(Displayable displayable, DeviceLabel title,
    //      int buttonCount,         
    boolean showButtons) {

        this.displayable = displayable;
        this.showButtons = showButtons;
        this.title = title;

        ApplicationManager manager = ApplicationManager.manager;

        setWidth(manager.screenWidth);
        setHeight(manager.screenHeight);

        if (title != null) {
            add(title);
            title.setHeight (title.getMinimumSize().height);
            title.setWidth (getWidth ());
        }

        buttons =
            new SoftButton[manager.getIntProperty("softButtons.count", 2)];

        for (int i = 0; i < buttons.length; i++) {
            buttons[i] = new SoftButton(this, i);
            if (showButtons) add (buttons [i]);
        }
    }

    public void paint(java.awt.Graphics g) {

        Vector toRun = null;

        if (displayable.display != null
            && displayable.display.callSerially.size() > 0) {
            toRun = displayable.display.callSerially;
            displayable.display.callSerially = new Vector();
        }

        super.paint(g);

        if (toRun != null)
            for (int i = 0; i < toRun.size(); i++)
                 ((Runnable) toRun.elementAt(i)).run();
    }

    public boolean keyPressed(int code, int modifiers) {
        if (code >= KeyEvent.VK_F1 && code < KeyEvent.VK_F1 + buttons.length) {
            System.out.println(
                "Function key F"
                    + (code - KeyEvent.VK_F1 + 1)
                    + " intercepted!");
            displayable.handleCommand(buttons[code - KeyEvent.VK_F1].command);
            return true;
        }
        else
            return super.keyPressed(code, modifiers);

    }

    void setMain(ScmComponent main) {
        if (this.main != null)
            remove(this.main);

        this.main = main;
        add(main);
        doLayout();
    }

    /*    void handleSoftButton(int index) {
    
            Command cmd = buttons[index].command;
    */



    void updateButtons() {

        int available = buttons.length;
        for (int i = 0; i < available; i++)
            buttons[i].setCommand(null);

        if (!showButtons && (displayable instanceof Canvas)) {
            buttons[0].setCommand(GAME_COMMAND1);
            available--;
            if (displayable.commands.size() == 0) {
                buttons[1].setCommand(GAME_COMMAND2);
                available--;
            }
        }
        else if (
            displayable.noSelectButton && displayable.selectButtonRequired) {
            buttons[buttons.length - 1].setCommand(SELECT_COMMAND);
            available--;
        }

        if (available < displayable.commands.size()) {

            buttons[(
                displayable.noSelectButton
                    && ((displayable instanceof List)
                        || (displayable instanceof Form)))
                ? 0
                : buttons.length - 1].setCommand(MENU_COMMAND);
            available--;
        }

        // assign remaining buttons by type

        boolean[] assigned = new boolean[displayable.commands.size()];
        for (int i = 0; i < displayable.commands.size(); i++) {
            Command cmd = (Command) displayable.commands.elementAt(i);
            String preferred =
                ApplicationManager.manager.properties.getProperty(
                    "commands.keys." + TYPES[cmd.type]);

            if (preferred != null && preferred.startsWith("SOFT")) {
                int index = Integer.parseInt(preferred.substring(4)) - 1;
                if (buttons[index].command == null) {
                    buttons[index].setCommand(cmd);
                    assigned[i] = true;
                }
            }
        }

        // assign remaining buttons by priority

        int cmdIndex = 0;
        for (int i = 0;
            i < buttons.length && cmdIndex < displayable.commands.size();
            i++)
            if (buttons[i].command == null && !assigned[cmdIndex])
                buttons[i].setCommand(
                    (Command) displayable.commands.elementAt(cmdIndex++));
    }

    public void doLayout() {
        
        ApplicationManager manager = ApplicationManager.manager;
        
        int y0 = 0;
        int y1 = getHeight ();
        
        for (int i = 0; i < getComponentCount (); i++) {
            ScmComponent c = getComponent (i);
            if (c == main) continue;
            
            if (c.getY() < getWidth () / 2)
                y0 = Math.max(y0, c.getY() + c.getHeight());
            else 
                y1 = Math.min(y1, c.getY());
        }
                    
        main.setBounds(0, y0+1, getWidth(), y1-y0-2);
        super.doLayout();
    }

}
