// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS: API Complete
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import org.me4se.impl.lcdui.*;

public class Font {

    static Font defaultFont;

    FontInfo info;

    public static final int STYLE_PLAIN = 0;
    public static final int STYLE_BOLD = 1;
    public static final int STYLE_ITALIC = 2;
    public static final int STYLE_UNDERLINED = 4;

    public static final int SIZE_SMALL = 8;
    public static final int SIZE_MEDIUM = 0;
    public static final int SIZE_LARGE = 16;

    public static final int FACE_MONOSPACE = 32;
    public static final int FACE_PROPORTIONAL = 64;
    public static final int FACE_SYSTEM = 0;

    int face;
    int style;
    int size;

    Font(int face, int style, int size) {

        Display.check();

        this.face = face;
        this.style = style;
        this.size = size;

        StringBuffer buf = new StringBuffer();
        String name;
        int points =
            javax
                .microedition
                .midlet
                .ApplicationManager
                .manager
                .awtContainer
                .getFont()
                .getSize();
        int awtStyle = 0;

        switch (face) {
            case FACE_SYSTEM :
                buf.append("system");
                break;
            case FACE_PROPORTIONAL :
                buf.append("proportional");
                break;
            case FACE_MONOSPACE :
                buf.append("monospace");
                break;
            default :
                throw new IllegalArgumentException();
        }

        switch (style & 3) {
            case STYLE_BOLD :
                buf.append(".bold");
                break;
            case STYLE_ITALIC :
                buf.append(".italic");
                break;
            case STYLE_BOLD | STYLE_ITALIC :
                buf.append(".bold.italic");
                break;
            default :
                buf.append(".plain");
        }

        switch (size) {
            case SIZE_MEDIUM :
                buf.append(".medium");
                break;
            case SIZE_SMALL :
                buf.append(".small");
                break;
            case SIZE_LARGE :
                buf.append(".large");
                break;
            default :
                throw new IllegalArgumentException();
        }

        info = FontInfo.getFontInfo(buf.toString());
    }

    public int charWidth(char c) {
        return info.font.charWidth(c);
    }

    public int charsWidth(char[] c, int ofs, int len) {
        return info.font.charsWidth(c, ofs, len);
    }

    public int getBaselinePosition() {
        return info.font.ascent;
    }

    public static Font getDefaultFont() {
        if (defaultFont == null)
            defaultFont = new Font(0, 0, 0);

        return defaultFont;
    }

    public int getFace() {
        return face;
    }

    public static Font getFont(int face, int style, int size) {
        return new Font(face, style, size);
    }

    public int getHeight() {
        return info.font.height;
    }

    public int getSize() {
        return size;
    }

    public int getStyle() {
        return style;
    }

    public boolean isBold() {
        return (style & STYLE_BOLD) != 0;
    }

    public boolean isUnderlined() {
        return (style & STYLE_UNDERLINED) != 0;
    }

    public boolean isItalic() {
        return (style & STYLE_ITALIC) != 0;
    }
    public boolean isPlain() {
        return style == 0;
    }

    public int stringWidth(String s) {
        return info.font.stringWidth(s);
    }

    public int substringWidth(String s, int ofs, int len) {
        return info.font.stringWidth(s.substring(ofs, ofs + len));
    }
}