// ME4SE - A MicroEdition Emulation for J2SE 
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// Contributors:
//
// STATUS:
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.lcdui;

import javax.microedition.midlet.*;
import org.me4se.impl.lcdui.*;

public class Graphics {

	public static final int HCENTER = 1;
	public static final int VCENTER = 2;
	public static final int LEFT = 4;
	public static final int RIGHT = 8;
	public static final int TOP = 16;
	public static final int BOTTOM = 32;

	public static final int BASELINE = 64;
	public static final int SOLID = 0;
	public static final int DOTTED = 1;

	public int translateX = 0;
	public int translateY = 0;

	public int strokeStyle = SOLID;

	boolean stale;

	java.awt.Graphics g;
	Font font;
	java.awt.Color awtColor = java.awt.Color.white;
	int midpColor = 0x0ffffff;
	Canvas canvas;

	Graphics(Canvas canvas, java.awt.Graphics g) {
		this.g = g;
		this.canvas = canvas;
		if (g == null)
			throw new RuntimeException("g==null");
		setFont(Font.getDefaultFont());
	}

	void checkStale() {
		if (stale && canvas != null) {
			canvas.component.paint(canvas.component.getGraphics());
			//	    ApplicationManager.manager.displayContainer.repaint ();
		}
	}

	public void drawArc(int x, int y, int w, int h, int sa, int aa) {
		g.drawArc(x, y, w, h, sa, aa);
		checkStale();
	}

	public void drawChars(
		char[] chars,
		int ofs,
		int len,
		int x,
		int y,
		int align) {
		drawString(new String(chars, ofs, len), x, y, align);
	}

	public void drawImage(Image img, int x, int y, int align) {


		if (align == 0)
			align = TOP | LEFT;

		switch (align & (TOP | BOTTOM | BASELINE | VCENTER)) {
			case TOP :
				break;
			case BOTTOM :
				y -= img.getHeight();
				break;
			case VCENTER :
				y -= img.getHeight() / 2;
				break;
			default :
				throw new IllegalArgumentException();
		}

		switch (align & (LEFT | RIGHT | HCENTER)) {
			case LEFT :
				break;
			case RIGHT :
				x -= img.getWidth();
				break;
			case HCENTER :
				x -= img.getWidth() / 2;
				break;
			default :
				throw new IllegalArgumentException();
		}


		while (!g.drawImage(img._image, x, y, null))
            ApplicationManager.waitForImage(img._image, img.name);
		//g.drawRect (x, y, img.getWidth (), img.getHeight ());

		//System.out.println ("drawimg stale: "+stale);
		checkStale();
	}

	public void drawLine(int x0, int y0, int x1, int y1) {

		////System.out.println ("enter drawline"+strokeStyle);

		if (strokeStyle == SOLID)
			g.drawLine(x0, y0, x1, y1);

		else {
			int dx = x1 - x0;
			int dy = y1 - y0;
			int steps = Math.max(Math.abs(dx), Math.abs(dy)) / 4;
			dx = (dx << 16) / steps;
			dy = (dy << 16) / steps;
			x0 = x0 << 16;
			y0 = y0 << 16;

			while (steps > 0) {
				g.drawLine(
					x0 >> 16,
					y0 >> 16,
					(x0 + dx) >> 16,
					(y0 + dy) >> 16);
				x0 += dx + dx;
				y0 += dy + dy;
				steps -= 2;
			}
		}

		//	//System.out.println ("leave drawline");

		checkStale();
	}

	public void drawString(String text, int x, int y, int align) {

		if (align == 0)
			align = TOP | LEFT;

		PhysicalFont fm = font.info.font;

		int cut = text.indexOf('\n');
		if (cut != -1) {
			drawString(text.substring(cut + 1), x, y + fm.height, align);
			text = text.substring(0, cut);
		}

		//y--;

		switch (align & (TOP | BOTTOM | BASELINE | VCENTER)) {
			case TOP :
				y += fm.ascent;
				break;
			case BOTTOM :
				y -= fm.descent;
				break;
			case BASELINE :
				break;
			case VCENTER :
				y = y + fm.ascent - fm.height / 2;
				break;
			default :
				throw new IllegalArgumentException();
		}

		switch (align & (LEFT | RIGHT | HCENTER)) {
			case LEFT :
				break;
			case RIGHT :
				x -= fm.stringWidth(text);
				break;
			case HCENTER :
				x -= fm.stringWidth(text) / 2;
				break;
			default :
				throw new IllegalArgumentException();
		}

		font.info.drawString(g, text, x, y);
		checkStale();
	}

	public void drawRect(int x, int y, int w, int h) {
		g.drawRect(x, y, w, h);
		checkStale();
	}

	public void drawRoundRect(int x, int y, int w, int h, int r1, int r2) {
		g.drawRoundRect(x, y, w, h, r1, r2);
		checkStale();
	}

	public void fillArc(int x, int y, int w, int h, int sa, int aa) {
		g.fillArc(x, y, w, h, sa, aa);
		checkStale();
	}

	public void fillRect(int x, int y, int w, int h) {
		g.fillRect(x, y, w, h);
		checkStale();
	}

	public void fillRoundRect(int x, int y, int w, int h, int r1, int r2) {
		g.fillRoundRect(x, y, w, h, r1, r2);
		checkStale();
	}

	public Font getFont() {
		return font;
	}

	public int getClipX() {
		java.awt.Rectangle r = g.getClipBounds();
		return r == null ? 0 : r.x;
	}

	public int getClipY() {
		java.awt.Rectangle r = g.getClipBounds();
		return r == null ? 0 : r.y;
	}

	public int getClipWidth() {
		java.awt.Rectangle r = g.getClipBounds();
		return r == null ? canvas.getWidth() : r.width;
	}

	public int getClipHeight() {
		java.awt.Rectangle r = g.getClipBounds();
		return r == null ? canvas.getHeight() : r.height;
	}

	public int getColor() {
		return midpColor;
	}

	public void setFont(Font font) {
		if (font == null)
			font = Font.getDefaultFont();
		//g.setFont(font.info.font);
		this.font = font;
	}

	public void translate(int x, int y) {
		translateX += x;
		translateY += y;

		g.translate(x, y);
	}

	public void clipRect(int x, int y, int w, int h) {
		g.clipRect(x, y, w, h);
	}

	public void setClip(int x, int y, int w, int h) {
		g.setClip(x, y, w, h);
	}

	public void setColor(int c) {

		midpColor = c;
		awtColor =
			new java.awt.Color(
				ApplicationManager.manager.getDeviceColor(c, true));

		if (g != null)
			g.setColor(awtColor);
	}

	public void setColor(int cr, int cg, int cb) {
		setColor((cr << 16) | (cg << 8) | cb);
	}

	public void setGrayScale(int gsc) {
		setColor(gsc | (gsc << 8) | (gsc << 16));
	}

	public void drawSubstring(
		String str,
		int offset,
		int len,
		int x,
		int y,
		int anchor) {

		drawString(str.substring(offset, offset + len - 1), x, y, anchor);
	}

	public void drawChar(char character, int x, int y, int anchor) {
		char characters[] = new char[1];
		characters[0] = character;
		drawString(new String(characters), x, y, anchor);
	}

	public int getBlueComponent() {
		return g.getColor().getBlue();
	}

	public int getGreenComponent() {
		return g.getColor().getGreen();
	}

	public int getRedComponent() {
		return g.getColor().getRed();
	}

	public int getGrayScale() {
		return getRedComponent();
	}

	public int getTranslateX() {
		return translateX;
	}

	public int getTranslateY() {
		return translateY;
	}

	public void setStrokeStyle(int style) {
		strokeStyle = style;
	}

	public int getStrokeStyle() {
		return strokeStyle;
	}
}
