// ME4SE - A MicroEdition Emulation for J2SE
//
// Copyright (C) 2001 Stefan Haustein, Oberhausen (Rhld.), Germany
//
// STATUS: API complete
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version. This program is
// distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
// License for more details. You should have received a copy of the
// GNU General Public License along with this program; if not, write
// to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

package javax.microedition.midlet;

import org.me4se.impl.gcf.*;
import org.me4se.impl.*;
import org.me4se.impl.jam.*;
import org.me4se.impl.lcdui.*;
import java.util.*;
import java.io.*;
import com.sixlegs.image.png.*;
import java.awt.Toolkit;
import java.awt.Dimension;
import java.net.*;
import org.kobjects.scm.*;

/** this class is needed *here* in order to be able to call the
    protected MIDlet startApp() method (etc.). It should perhaps be
    hidden from the documentation. */

public class ApplicationManager {

	public static ApplicationManager manager;

    public MIDlet active;

    public int colorCount = 256;
    public boolean isColor = true;
    public Properties properties;
    public java.awt.Color bgColor = java.awt.Color.white;
    public ScmContainer skin;
    // custom fixed device screen elements 
    public Hashtable custom = new Hashtable();
    public Hashtable icons = new Hashtable();

    public JadFile jadFile = new JadFile();

    public java.awt.Image offscreen;
    public java.awt.Dimension offscreenSize;
    public ScmContainer displayContainer;
    public ScmWrapper wrapper = new ScmWrapper ();
    public java.awt.Frame frame;
    public java.awt.Container awtContainer;
    
    public int screenWidth;
    public int screenHeight;

    public String documentBase;

    public boolean isApplet;

    public ClassLoader classLoader;

    /** 
     * loaads an image from the given file name, using the 
     * openInputStream() method. If the file extension is PNG or png,
     * the image is loaded using the sixlegs png library. */

    public java.awt.Image getImage(String fileName) throws IOException {
        //        System.out.println ("getImage: "+fileName);
        if (fileName.endsWith(".png") || fileName.endsWith(".PNG"))
            return getPngImage(openInputStream(fileName));
        else {

            InputStream is = openInputStream(fileName);
            if (is == null)
                throw new RuntimeException("null stream opening: " + fileName);
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            while (true) {
                int count = is.read(buffer);
                if (count <= 0)
                    break;
                os.write(buffer, 0, count);

            }
            is.close();
            os.close();

            if (os.size() == 0)
                throw new RuntimeException("empty image file: " + fileName);

            java.awt.Image img =
                Toolkit.getDefaultToolkit().createImage(os.toByteArray());
            // waitForImage (img, fileName);
            return img;
            /*
                String test = fileName.toLowerCase ();
                if (test.startsWith("http://") || test.startsWith ("file://")) {
                    return tk.getImage(new URL (fileName));
                }
                else if (documentBase == null) {  
                    return tk.getImage (fileName);
                }
                else {
                    return tk.getImage (new URL (documentBase.toString () + fileName));
            } */
        }
    }

    /** 
     * opens an input stream on the given resource; if the path is
     * not absolute and not an URL, the documentBase is taken into
     * account. */

    public InputStream openInputStream(String fileName) throws IOException {
        //            System.out.println ("open stream: "+fileName);

        fileName = concatPath(documentBase, fileName);

        //     System.out.println ("concatenated: "+fileName);

        if (isUrl(fileName)) {
            //System.out.println("opening url: " + fileName);
            return new URL(fileName).openStream();
        }
        // try resource

        try {
            InputStream result =
                ApplicationManager.class.getResourceAsStream(fileName);
            if (result != null) {
                System.out.println("ressource opended: " + fileName);
                return result;
            }
        }
        catch (Exception e) {
        }

        // file otherwise
        System.out.println("opening file: " + fileName);

        return new FileInputStream(fileName);
    }

    /** determines whether path starts with http:// or file:// */

    public static boolean isUrl(String path) {
        String test = path.toLowerCase();
        return test.startsWith("http://") || test.startsWith("file:");
    }

    /** if file is absolute (starts with /, http:// or file://, 
     * or the base path is null, file is returned; otherwise the 
     * last index of / or \ is searched in base and 
     * the strings are concatenated */

    public static String concatPath(String base, String file) {
        if (base == null || file.startsWith("/") || file.startsWith ("\\") || (file.length () >= 2 && file.charAt(1) == ':') || isUrl(file))
            return file;

        int cut = Math.max(base.lastIndexOf('/'), base.lastIndexOf('\\'));

        return base.substring(0, cut + 1) + file;
    }

    public static java.awt.Image getPngImage(InputStream is)
        throws IOException {
        PngImage pi = new com.sixlegs.image.png.PngImage(is);
        pi.getEverything();
        is.close();
        return java.awt.Toolkit.getDefaultToolkit().createImage(pi);
    }

    public static void waitForImage(java.awt.Image image, String info) {
        //System.out.println ("enter waitforimage" +manager.displayContainer);

        //System.out.println("wait for img: " + info);
        java.awt.MediaTracker tracker =
            new java.awt.MediaTracker(ApplicationManager.manager.awtContainer);
        tracker.addImage(image, 0);
        try {
            tracker.waitForID(0);
        }
        catch (InterruptedException ie) {
        }

        if (tracker.isErrorAny())
            throw new RuntimeException("error loading image " + info);
        //System.out.println("image ok: " + info);
        //System.out.println ("leave waitforimage");
    }

    public ApplicationManager(
        java.applet.Applet applet,
        Properties properties) {

        this.properties = properties;

        //FontInfo.cache = new Hashtable ();

        if (manager != null)
            manager.destroy();

        manager = this;

        if (applet != null) {
            isApplet = true;
            applet.setLayout(new java.awt.BorderLayout());

            documentBase = applet.getDocumentBase().toString();
            awtContainer = applet;
        
            screenWidth = applet.getSize().width;
            screenHeight = applet.getSize().height;
        }
        else {
            frame = new java.awt.Frame("MIDlet");

			
            //screenWidth = getIntProperty("screen.width", 150);
            //screenHeight = getIntProperty ("screen.height", 200);        

			// Hi Stefan. Ist das Okay so? Die Kommandozeilen - Parameter fr
			// Hhe und Breite des Fensters werden nun noch vor denen vom Properties
			// File ausgewertet. Alles im !applet Mode.

			screenWidth = (properties.get("width") != null) 
				? Integer.parseInt((String)properties.get("width")) 
				: getIntProperty("screen.width", 150);
			
			screenHeight = (properties.get("height") != null) 
				? Integer.parseInt((String)properties.get("height")) 
				: getIntProperty("screen.height", 200);


            // *********************************
            // * Set microedition.* properties *
            // *********************************

            String os = System.getProperty("os.name");
			String osArch = System.getProperty("os.arch");
            String platform = "ME4SE_J2SE";

            if (os != null) {
                if (os.equals("EPOC")) {
                    platform = "ME4SE_EPOC";
                }
                else if (os.equals("Windows CE")) {
                    platform = "ME4SE_WINCE";
                }
                else if (os.equals("Linux") && osArch.equals("arm")) {
            		platform = "ME4SE_ZAURUS";
            	}
            }

            Properties sysProps = System.getProperties();

            sysProps.put("microedition.platform", platform);
            sysProps.put("microedition.encoding", "ISO8859_1");
            sysProps.put(
                "microedition.locale",
                Locale.getDefault().getCountry());
            sysProps.put("microedition.configuration", "CLDC-1.0");
            sysProps.put("microedition.profiles", "MIDP-1.0");

            System.setProperties(sysProps);

            if (platform.equals("ME4SE_WINCE")) {
                Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
                screenWidth = dim.width;
                screenHeight = dim.height - 25;
            }
            else if (platform.equals("ME4SE_LINUX")) {
            	screenWidth = 235;
            	screenHeight = 280;
            }
            else if (platform.equals("ME4SE_EPOC")) {
                Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
                screenWidth = dim.width;
                screenHeight = dim.height;
            }
            
            frame.addWindowListener(new java.awt.event.WindowAdapter() {
                public void windowClosing(java.awt.event.WindowEvent ev) {
                    destroy();
                    System.exit(0);
                }
            });

            awtContainer = frame;
            //frame.show ();
        }

        if (properties.getProperty("skin") != null) {
            try {
                skin =
                    (ScmContainer) Class
                        .forName("org.me4se.impl.skins.Skin")
                        .newInstance();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            displayContainer = skin;

            // access to skin properties available now
            
            screenWidth = getIntProperty("screen.width", screenWidth);
            screenHeight = getIntProperty ("screen.height", screenHeight);        
        }
        else {
            displayContainer = new ScmContainer() {
               public Dimension getMinimumSize () {
                return new Dimension (screenWidth, screenHeight);
               }
            };
            displayContainer.setBackground(new java.awt.Color (0x0ffffff));
        }

        wrapper.setComponent(displayContainer);
	//	wrapper.setFocusable (true);

        awtContainer.add(wrapper, java.awt.BorderLayout.CENTER);

        if (skin != null && frame != null) {
            frame.pack();
            frame.show();
            frame.setResizable (false);
        }
        else if (isApplet) {
            wrapper.invalidate ();
            awtContainer.validate ();	    
        }

         // the MIDP API does not permit runtime size changes
            // runtime size changes may result in unpredictable
            // behaviour
    }

    /** not called if MIDlet provides a main method */

    public void launch() {

        try {
            if (properties.getProperty("socketProxyHost") != null) {
                ConnectionImpl connectionImpl =
                    (ConnectionImpl) Class
                        .forName("org.me4se.impl.ConnectionImpl_socket")
                        .newInstance();

                connectionImpl.initialise(properties);
            }

            if (properties.getProperty("httpProxyHost") != null) {
                ConnectionImpl connectionImpl =
                    (ConnectionImpl) Class
                        .forName("org.me4se.impl.ConnectionImpl_http")
                        .newInstance();
                connectionImpl.initialise(properties);
            }
        }
        catch (Exception e) {
            throw new RuntimeException("Error initializing proxy:" + e);
        }

        if (!isApplet && System.getProperty("comm.enable") != null) {
            try {
                ConnectionImpl connectionImpl =
                    (ConnectionImpl) Class
                        .forName("org.me4se.impl.gcf.ConnectionImpl_comm")
                        .newInstance();
                connectionImpl.initialise(null);
            }
            catch (Exception e) {
                throw new RuntimeException(
                    "Error initializing CommConnection:" + e);
            }

        }

        String jadUrl = properties.getProperty("jad");

        if (jadUrl == null) {
            String midlet = properties.getProperty("MIDlet");

            /**
             * If no MIDlet to start has been specified, but we have a
             * Java Application Manager set, start this one.
             */
            if (midlet == null)
				midlet = properties.getProperty("jam");

			if (midlet == null)
                throw new RuntimeException("No MIDlet or jad specified");

            jadFile.add("MIDlet-1: MIDlet,," + midlet);
        }
        else {
            try {
                jadFile.load(openInputStream(jadUrl));
                System.out.println("jad loaded!");
            }
            catch (Exception e) {
                throw new RuntimeException("Error opening JAD file:" + e);
            }
        }

        // care about rms location

        if (isApplet) {
            //System.out.println ("setRmsDir to null (applet)");
            org.me4se.impl.rms.RecordStoreImpl_file.rmsDir = null;
        }
        else {
            File rmsDir = new File(System.getProperty("rms.home", "rms"));
            rmsDir.mkdirs();
            org.me4se.impl.rms.RecordStoreImpl_file.rmsDir = rmsDir;
        }

        try {
            //    System.out.println ("name:" +name);
            if (jadFile.getMIDletCount() == 1)
                active =
                    ((MIDlet) Class
                        .forName(jadFile.getMIDlet(1).getClassName())
                        .newInstance());
            else
                active = new MIDletChooser();
            //  System.out.println ("midlez:" +midlet);
        }
        catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e.toString());
        }
    }
    

    public int getIntProperty(String name, int dflt) {
        try {
            return Integer
                .decode(
                    (String) ApplicationManager.manager.properties.get(name))
                .intValue();
        }
        catch (Exception e) {
            return dflt;
        }
    }

    public int getDeviceColor(int color, boolean step) {

        if (isColor)
            return color;

        double grayscale =
            (((color >> 16) & 255) + ((color >> 8) & 255) + (color & 255))
                / (3.0 * 255.0);

        if (step) {
            //System.out.println ("grayscale: "+grayscale+" cc: "+colorCount);

            grayscale = (int) ((colorCount - 1) * grayscale + 0.5);

            //System.out.println ("grayscale discr: "+grayscale);
            grayscale /= (colorCount - 1);

            //System.out.println ("grayscale adj: "+grayscale);
        }

        java.awt.Color bg = ApplicationManager.manager.bgColor;

        int r = (int) grayscale * bg.getRed();
        int g = (int) grayscale * bg.getGreen();
        int b = (int) grayscale * bg.getBlue();

        return (color & 0x0ff000000) | (r << 16) | (g << 8) | b;
    }

    void setRmsDir(String dflt) {

        //System.out.println ("setRmsDir");

    }

    public void start() {
        try {
            if (active != null)
                active.startApp();
        }
        catch (MIDletStateChangeException e) {
        }
    }

    public void pause() {

        if (active != null)
            active.pauseApp();
    }

    public void destroy() {
        try {
            if (active != null)
                active.destroyApp(true);
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        active = null;
    }

    /**
	 * Instantiates and launches the given MIDlet.
     */
	public void startMIDlet(String name) throws InstantiationException, ClassNotFoundException, IllegalAccessException {
        MIDlet midlet = null;

        if (classLoader != null) {
            midlet = ((MIDlet) classLoader.loadClass(name).newInstance());
        }
        else {
            midlet = ((MIDlet) Class.forName(name).newInstance());
        }

        // Throw away everything in the current DisplayContainer. Tell
        // the ApplicationManager that the newly created MIDlet is the
        // active one. Then run it.
        //        displayContainer.removeAll();
        //displayContainer.setLayout(new java.awt.BorderLayout());
        //displayContainer.setBackground(bgColor);

        active = midlet;
        start();
    }
}

/*
 * $Log: ApplicationManager.java,v $
 * Revision 1.62  2002/09/09 10:23:12  mkroll
 * Fixed sizing on IPaq.
 *
 * Revision 1.61  2002/08/23 12:01:39  mkroll
 * Changes behavior of initial window size if the MIDletRunner is started as application.
 *
 * Revision 1.60  2002/08/15 12:23:46  haustein
 * select button/focus issue fixed, path concatenation issue fixed
 *
 * Revision 1.59  2002/08/10 14:20:19  mkroll
 * Removed System.out.println (Debug information).
 *
 * Revision 1.58  2002/08/10 14:18:41  mkroll
 * Removed System.out.println (Debug information).
 *
 * Revision 1.57  2002/08/08 19:26:02  mkroll
 * text field and window size fixes
 *
 * Revision 1.56  2002/08/05 22:01:40  mkroll
 * Added ME4SE_Zaurus as microedition.platform property while running ME4SE on a Zaurus.
 *
 * Revision 1.55  2002/08/05 18:26:38  mkroll
 * Changed useCommPorts to comm.enable
 *
 * Revision 1.54  2002/08/01 18:23:29  mkroll
 * changed gcf codebase in class by name call for comm port initialization.
 *
 * Revision 1.53  2002/07/31 22:37:43  haustein
 * html changes, canvas repaint fix
 *
 * Revision 1.52  2002/07/30 22:47:18  haustein
 * some constraint support added
 *
 * Revision 1.51  2002/07/29 20:09:51  haustein
 * unskinned background issue fixed
 *
 * Revision 1.50  2002/07/28 20:43:59  haustein
 * branch merged
 *
 * Revision 1.47.2.3  2002/07/27 20:00:31  haustein
 * merged
 *
 * Revision 1.47.2.2  2002/07/27 18:05:56  haustein
 * merged
 *
 * Revision 1.47.2.1  2002/07/21 23:08:21  haustein
 * branch cvs backup, not completely working yet
 *
 * Revision 1.47  2002/07/16 17:54:24  pleumann
 * Fehler im MIDletChooser korrigiert
 *
 * Revision 1.46  2002/07/15 22:56:24  mkroll
 * Added last fixes after moving CommConnection to org.me4se.impl.gcf
 *
 * Revision 1.45  2002/07/15 22:46:52  haustein
 * moved jad and chooser back, moved rms and gcf stuff to appropriate subpackages
 *
 * Revision 1.44  2002/07/15 21:06:56  pleumann
 * Loading images from JAR works. Now Joerg is going to suspend mode.
 *
 * Revision 1.43  2002/07/15 19:56:44  pleumann
 * Moved JadFile and MIDletChooser to jam package
 *
 * Revision 1.42  2002/07/15 14:23:10  haustein
 * skinned version seems to run (more) stable on osX now
 *
 * Revision 1.41  2002/07/15 11:44:38  haustein
 * some output added because blocking problems on apple osx
 *
 * Revision 1.40  2002/07/15 10:12:53  haustein
 * source code formatting fixed
 *
 * Revision 1.39  2002/07/15 10:08:11  haustein
 * file loading improvements, path concatenation method added
 *
 * Revision 1.38  2002/07/15 09:07:45  mkroll
 * Added an if clause arround comm initialization.
 *
 * Revision 1.37  2002/07/14 22:45:26  haustein
 * applet property access fix, resource access  fixes, rms applet fix
 *
 * Revision 1.36  2002/07/14 13:38:42  haustein
 * font fixes, ressource loading unification
 *
 * Revision 1.35  2002/07/11 22:15:53  mkroll
 * MIDlet size is set to fullscreen on WINCE and EPOC devices. J2SE is still 200x150 pixels in size.
 *
 * Revision 1.34  2002/07/11 21:30:57  mkroll
 * Added microedition.platform *.encoding *.locale *.configuration and *.profiles if running as application.
 *
 * Revision 1.33  2002/07/11 20:22:58  haustein
 * isApplet moved from MIDletRunner to ApplicationManager, corresp. code adoptions
 *
 * Revision 1.32  2002/07/10 14:39:55  haustein
 * sebastians JAD support replaced by joergs class
 *
 * Revision 1.31  2002/07/10 14:35:41  haustein
 * adoptions to new jad file provided by joerg
 *
 * Revision 1.30  2002/07/06 21:29:45  haustein
 * some changes for better bitmap font support in xme4se
 *
 * Revision 1.29  2002/06/20 23:18:47  mkroll
 * Added support for CommConnection introduced by PDAP.
 *
 * Revision 1.28  2002/06/06 22:10:40  haustein
 * jdk1.4 compatibility fix, window fix
 *
 * Revision 1.27  2002/04/18 17:06:22  mkroll
 * Added functionality to specify rms.home as VM parameter for instance -Drms.home=C:\temp\
 *
 */
