/*
 * $Id:$
 */

package org.me4se.impl.jam;

import java.util.Properties;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

import org.me4se.impl.*;
import java.awt.FileDialog;
import java.io.*;

/**
 * @author Michael Kroll, michael.kroll@trantor.de
 * @author Jrg Pleumann, joerg.pleumann@trantor.de
 */

public class MIDletSuiteManager extends MIDlet implements CommandListener {

	private class RMSFilter implements FilenameFilter {

		public boolean accept(File dir, String name) {
			name = name.toLowerCase();
			return name.endsWith(".rms");
		}
	}

	class RemoteServerScreen extends Form {

		TextField url = new TextField("Remote Server:", "", 40, TextField.ANY);
		public Command OTA_OK = new Command("Ok", Command.OK, 0);
		public Command OTA_BACK = new Command("Back", Command.BACK, 0);
		MIDlet parent;

		public RemoteServerScreen(CommandListener listener) {
			super("Enter Remote Server");
			this.parent = parent;
			append(url);
			addCommand(OTA_OK);
			addCommand(OTA_BACK);
			setCommandListener(listener);
		}

		public String getRemoteServer() {
			return url.getString();
		}
	}

	private MIDletDatabase database;
	private RemoteServerScreen remoteScreen;
    private static JarClassLoader classLoader = new JarClassLoader();
    	
	Command exitCmd = new Command("Exit", Command.EXIT, 0);
	Command openCmd = new Command("Open", Command.SCREEN, 0);
	Command installFileCmd = new Command("Install File", Command.SCREEN, 0);
	Command installRemoteCmd = new Command("Install Remote", Command.SCREEN, 0);
	Command removeCmd = new Command("Remove", Command.SCREEN, 0);
	Command infoCmd = new Command("Info", Command.SCREEN, 0);
	Command aboutCmd = new Command("About", Command.SCREEN, 0);

	List midletList = new List("Installed MIDlets", List.IMPLICIT);
	Display display;

	public MIDletSuiteManager() {
		midletList.addCommand(exitCmd);
		midletList.addCommand(openCmd);
		midletList.addCommand(installFileCmd);
		midletList.addCommand(installRemoteCmd);
		midletList.addCommand(removeCmd);
		midletList.addCommand(infoCmd);
		midletList.addCommand(aboutCmd);

		midletList.setCommandListener(this);

		database = new MIDletDatabase(System.getProperty("jam.homedir", "c:\\ME4SE-JAM"));

		String[] installedSuites = database.getSuiteList();
		for (int i = 0; i < installedSuites.length; i++) {
			midletList.append(installedSuites[i], null);
		}

	}

	private void refreshMIDletList() {
		for (int i = midletList.size() - 1; i >= 0; i--) {
			midletList.delete(i);
		}

		String[] installedSuites = database.getSuiteList();

		for (int i = 0; i < installedSuites.length; i++) {
			midletList.append(installedSuites[i], null);
		}
	}

	public void startApp() {
		display = Display.getDisplay(this);
		display.setCurrent(midletList);
	}

	public void commandAction(Command c, Displayable d) {
		if (c == exitCmd) {
			notifyDestroyed();
		}
		else if (c == installFileCmd) {
			FileDialog fileDialog = new FileDialog(new java.awt.Frame(), "Select JAD file", FileDialog.LOAD);

			fileDialog.setVisible(true);

			if (fileDialog.getFile() != null) {
				if (fileDialog.getFile().endsWith(".jad")) {
					try {
						String jadName = fileDialog.getDirectory() + File.separatorChar + fileDialog.getFile();
						database.installSuite(jadName);
					}
					catch (IOException ioe) {
						Alert alert = new Alert("JAD Error:");
						alert.setString("An Error occured opening the JAD file: " + ioe.getMessage());
						display.setCurrent(alert);

					}
				}
				else {
					Alert alert = new Alert("File Selection Error:");
					alert.setString("No JAD file selected. Please select a vaild JAD file.");
					display.setCurrent(alert);
				}
				refreshMIDletList();
			}
		}
		else if (c == removeCmd) {
			// add a confirmation screen here before the suite is deleted.
			int selectedSuite = midletList.getSelectedIndex();
			database.removeSuite(selectedSuite);
			refreshMIDletList();

		}
		else if (c == openCmd) {

			int selectedSuiteIdx = midletList.getSelectedIndex();
			String selectedSuiteName = midletList.getString(selectedSuiteIdx);
			JadFile jad = database.getSuite(selectedSuiteIdx);

			String path = jad.getValue("MIDlet-Jar-Url");
			
			java.util.Properties props = System.getProperties();
			props.put("rms.home", path.substring(0, path.lastIndexOf("\\")));
			System.setProperties(props);

            classLoader.setJarFile(path);

			ApplicationManager.manager.jadFile = jad;

            try {
                if (jad.getMIDletCount() != 1) {
                    ApplicationManager.manager.startMIDlet("org.me4se.impl.MIDletChooser");
                }
                else {
                    ApplicationManager.manager.startMIDlet(jad.getMIDlet(1).getClassName());
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
		}
		else if (c == installRemoteCmd) {

			remoteScreen = new RemoteServerScreen(this);
			display.setCurrent(remoteScreen);
 
		}
		else if (c == infoCmd) {

			int selectedSuiteIdx = midletList.getSelectedIndex();
			String selectedSuiteName = midletList.getString(selectedSuiteIdx);
			JadFile jad = database.getSuite(selectedSuiteIdx);

			Alert info = new Alert(selectedSuiteName + " Info:");

			String jarSize = "Jar-Size = " + jad.getValue("MIDlet-Jar-Size") + " bytes";

			String path = jad.getValue("MIDlet-Jar-Url");
			path = path.substring(0, path.lastIndexOf("\\"));
			
			File f = new File (path);
			String[] rmsFiles = f.list(new RMSFilter());

			long rmsSize = 0L;			
			
			for (int i = 0; i < rmsFiles.length; i++) {
				rmsSize += new File(rmsFiles[i]).length();
			}

			String rmsSizeString = "RMS Size = " + rmsSize + " bytes";
			String jarVendor = "Vendor = " + jad.getValue("MIDlet-Vendor");
			String jarVersion = "Version = " + jad.getValue("MIDlet-Version");

			info.setString(jarSize + "\n" + rmsSizeString + "\n" + jarVendor + "\n" + jarVersion);
			display.setCurrent(info);
		} else if (c == aboutCmd) {
			Alert alert = new Alert("ME4SE MIDlet Suite Manager.");
			alert.setString("Implemented by Michael Kroll and Jrg Pleumann. \n {michael.kroll, joerg.pleumann}@trantor.de, 2002");
			display.setCurrent(alert);

		}
		else if (c.getLabel().equals("Ok") && d == remoteScreen) {
			display.setCurrent(midletList);
		}
		else if (c.getLabel().equals("Back") && d == remoteScreen) {
			display.setCurrent(midletList);
		}
	}

	public void pauseApp() {
	}

	public void destroyApp(boolean b) {
	}
    
    public static void main(String[] argv) {

        Properties param = new Properties();
        String midlet = null;
        String jadfile = null;

        for (int i = 0; i < argv.length; i++) {

            if (argv[i].startsWith("-")) {
                param.put(argv[i].substring(1), argv[i + 1]);
                i++;
            }

            else if (argv[i].indexOf(".jad") == argv[i].length() - 4)
                param.put("jad", argv[i]);
            else
                param.put("MIDlet", argv[i]);
        }

        if (param.get("MIDlet") == null && param.get("jad") == null) {
            param.put("jam", "org.me4se.impl.jam.MIDletSuiteManager");
            param.put("MIDlet", "org.me4se.impl.jam.MIDletSuiteManager");
        }

        new ApplicationManager(null, param).launch();
        ApplicationManager.manager.classLoader = classLoader;
        ApplicationManager.manager.start();
    }
}

/*
 * $Log:$
 */